﻿using BveTypes.ClassWrappers;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TrackPos
{
    internal class TrackPlot
    {
        private List<List<double>> trackPosList = new List<List<double>>();

        private List<double> trackPos_X = new List<double>();
        private List<double> trackPos_Y = new List<double>();
        private List<double> trackPos_Z = new List<double>();
        private List<double> trackPos_dist = new List<double>();

        private double min_X, min_Y, min_Z, min_dist, max_X, max_Y, max_Z, max_dist;
        private double center_X, center_Y, center_Z, center_dist;
        private double magnif = 1.0, magnif_vert_h = 1.0, magnif_vert_v = 1.0;
        private double vehiclepos_X = 0.0, vehiclepos_Y = 0.0, vehiclepos_Z = 0.0, vehiclepos_kp = 0.0, vehiclepos_curvature = 0.0;

        private Bitmap trackimg = new Bitmap(255, 255);
        private Bitmap trackimg_vert = new Bitmap(255, 65);

        public TrackPlot()
        {
        }

        public void getTrackPos(MyTrack myTrack, double lastpos)
        {
            for (double i = 0; i < lastpos; i = i + 25)
            {
                this.trackPosList.Add(new List<double> { i, myTrack.GetPosition(i, 0)[0], myTrack.GetPosition(i, 0)[1], myTrack.GetPosition(i, 0)[2] });
                this.trackPos_dist.Add(i);
                this.trackPos_X.Add(myTrack.GetPosition(i, 0)[0]);
                this.trackPos_Y.Add(myTrack.GetPosition(i, 0)[1]);
                this.trackPos_Z.Add(myTrack.GetPosition(i, 0)[2]);
            }

            min_X = this.trackPos_X.Min();
            min_Y = this.trackPos_Y.Min();
            min_Z = this.trackPos_Z.Min();
            min_dist = this.trackPos_dist.Min();

            max_X = this.trackPos_X.Max();
            max_Y = this.trackPos_Y.Max();
            max_Z = this.trackPos_Z.Max();
            max_dist = this.trackPos_dist.Max();

            center_X = (min_X + max_X) / 2;
            center_Y = (min_Y + max_Y) / 2;
            center_Z = (min_Z + max_Z) / 2;
            center_dist = (min_dist + max_dist) / 2;

            if (max_X - min_X > max_Z - min_Z)
            {
                magnif = 200.0 / (max_X - min_X);
            }
            else
            {
                magnif = 200.0 / (max_Z - min_Z);
            }

            magnif_vert_h = 200.0/(max_dist-min_dist);
            magnif_vert_v = 50.0/(max_Y - min_Y);
        }

        public void getTrackPosList(List<List<double>> trackPosList_v)
        {
            trackPosList = trackPosList_v;
        }

        public void setVehiclePos(double x, double y, double z, double kp, double curvature)
        {
            this.vehiclepos_X = x;
            this.vehiclepos_Y = y;
            this.vehiclepos_Z = z;
            this.vehiclepos_kp = kp;
            this.vehiclepos_curvature = curvature;
        }

        public void drawTrack()
        {
            Graphics g = Graphics.FromImage(trackimg);
            Pen pen = new Pen(Color.Blue, 1);

            List<double> prevPos = this.trackPosList[0];
            foreach (List<double> trackPos in this.trackPosList)
            {
                g.DrawLine(pen,
                    (float)((prevPos[3]) * magnif) + 0,
                    (float)((prevPos[1]) * magnif) + 100,
                    (float)((trackPos[3]) * magnif) + 0,
                    (float)((trackPos[1]) * magnif) + 100);

                prevPos = trackPos;
            }
            pen.Dispose();
            g.Dispose();
        }

        public void drawTrackVert()
        {
            Graphics g = Graphics.FromImage(trackimg_vert);
            Pen pen = new Pen(Color.Blue, 1);

            List<double> prevPos = this.trackPosList[0];
            foreach (List<double> trackPos in this.trackPosList)
            {
                g.DrawLine(pen,
                    (float)((prevPos[0] - min_dist) * magnif_vert_h),
                    (float)(55-(prevPos[2] - min_Y) * magnif_vert_v),
                    (float)((trackPos[0] - min_dist) * magnif_vert_h),
                    (float)(55-(trackPos[2] - min_Y) * magnif_vert_v));

                prevPos = trackPos;
            }
            pen.Dispose();
            g.Dispose();
        }

        public Bitmap drawVehiclePos()
        {
            Bitmap img = new Bitmap(trackimg);
            Graphics g = Graphics.FromImage(img);
            Pen penR = new Pen(Color.Red, 2);
            Font font = new Font("MS UI Gothic", 9);

            g.DrawEllipse(penR, (float)((vehiclepos_Z) * magnif) - 1, (float)((vehiclepos_X) * magnif) + 100 - 1, 2, 2);
            g.DrawString(String.Format("KP: {0:#.#}\nR: {1:#.#}", this.vehiclepos_kp, 1 / this.vehiclepos_curvature), font, Brushes.Black, (float)((vehiclepos_Z) * magnif) - 1, (float)((vehiclepos_X) * magnif) + 100 - 1);
            penR.Dispose();
            font.Dispose();
            g.Dispose();

            return img;
        }

        public Bitmap drawVehiclePosVert()
        {
            Bitmap img = new Bitmap(trackimg_vert);
            Graphics g = Graphics.FromImage(img);
            Pen penR = new Pen(Color.Red, 2);
            Font font = new Font("MS UI Gothic", 9);

            g.DrawEllipse(penR,
                    (float)((vehiclepos_kp - min_dist) * magnif_vert_h)-1,
                    (float)(55 - (vehiclepos_Y - min_Y) * magnif_vert_v)-1,
                    2, 2);
            g.DrawString(String.Format("GR: {0:#.#}", 0.0), font, Brushes.Black, (float)((vehiclepos_kp - min_dist) * magnif_vert_h) - 1,(float)(55 - (vehiclepos_Y - min_Y) * magnif_vert_v) - 1);
            penR.Dispose();
            font.Dispose();
            g.Dispose();

            return img;
        }
    }
}
